@@ -7,14 +7,14 @@
 //!
 //! The main modules are:
 //!
-//! - [`client`](crate::client) with the Kubernetes [`Client`](crate::Client) and its layers
-//! - [`config`](crate::config) for cluster [`Config`](crate::Config)
-//! - [`api`](crate::api) with the generic Kubernetes [`Api`](crate::Api)
-//! - [`derive`](kube_derive) with the [`CustomResource`](crate::CustomResource) derive for building controllers types
-//! - [`runtime`](crate::runtime) with a [`Controller`](crate::runtime::Controller) / [`watcher`](crate::runtime::watcher()) / [`reflector`](crate::runtime::reflector::reflector) / [`Store`](crate::runtime::reflector::Store)
-//! - [`core`](crate::core) with generics from `apimachinery`
+//! - [`client`] with the Kubernetes [`Client`] and its layers
+//! - [`config`] for cluster [`Config`]
+//! - [`api`] with the generic Kubernetes [`Api`]
+//! - [`derive`](kube_derive) with the [`CustomResource`] derive for building controllers types
+//! - [`runtime`] with a [`Controller`](crate::runtime::Controller) / [`watcher`](crate::runtime::watcher()) / [`reflector`](crate::runtime::reflector::reflector) / [`Store`](crate::runtime::reflector::Store)
+//! - [`core`] with generics from `apimachinery`
 //!
-//! You can use each of these as you need with the help of the [exported features](https://github.com/kube-rs/kube-rs/blob/master/kube/Cargo.toml#L18).
+//! You can use each of these as you need with the help of the [exported features](https://github.com/kube-rs/kube/blob/main/kube/Cargo.toml#L18).
 //!
 //! # Using the Client
 //! ```no_run
@@ -30,7 +30,7 @@
 //!     // Read pods in the configured namespace into the typed interface from k8s-openapi
 //!     let pods: Api<Pod> = Api::default_namespaced(client);
 //!     for p in pods.list(&ListParams::default()).await? {
-//!         println!("found pod {}", p.name());
+//!         println!("found pod {}", p.name_any());
 //!     }
 //!     Ok(())
 //! }
@@ -39,7 +39,7 @@
 //! For details, see:
 //!
 //! - [`Client`](crate::client) for the extensible Kubernetes client
-//! - [`Api`](crate::Api) for the generic api methods available on Kubernetes resources
+//! - [`Api`] for the generic api methods available on Kubernetes resources
 //! - [k8s-openapi](https://docs.rs/k8s-openapi/*/k8s_openapi/) for documentation about the generated Kubernetes types
 //!
 //! # Using the Runtime with the Derive macro
@@ -48,22 +48,21 @@
 //! use schemars::JsonSchema;
 //! use serde::{Deserialize, Serialize};
 //! use serde_json::json;
-//! use validator::Validate;
 //! use futures::{StreamExt, TryStreamExt};
 //! use k8s_openapi::apiextensions_apiserver::pkg::apis::apiextensions::v1::CustomResourceDefinition;
 //! use kube::{
-//!     api::{Api, DeleteParams, ListParams, PatchParams, Patch, ResourceExt},
+//!     api::{Api, DeleteParams, PatchParams, Patch, ResourceExt},
 //!     core::CustomResourceExt,
 //!     Client, CustomResource,
 //!     runtime::{watcher, WatchStreamExt, wait::{conditions, await_condition}},
 //! };
 //!
 //! // Our custom resource
-//! #[derive(CustomResource, Deserialize, Serialize, Clone, Debug, Validate, JsonSchema)]
+//! #[derive(CustomResource, Deserialize, Serialize, Clone, Debug, JsonSchema)]
 //! #[kube(group = "clux.dev", version = "v1", kind = "Foo", namespaced)]
 //! pub struct FooSpec {
 //!     info: String,
-//!     #[validate(length(min = 3))]
+//!     #[schemars(length(min = 3))]
 //!     name: String,
 //!     replicas: i32,
 //! }
@@ -87,10 +86,10 @@
 //!
 //!     // Watch for changes to foos in the configured namespace
 //!     let foos: Api<Foo> = Api::default_namespaced(client.clone());
-//!     let lp = ListParams::default();
-//!     let mut apply_stream = watcher(foos, lp).applied_objects().boxed();
+//!     let wc = watcher::Config::default();
+//!     let mut apply_stream = watcher(foos, wc).applied_objects().boxed();
 //!     while let Some(f) = apply_stream.try_next().await? {
-//!         println!("saw apply to {}", f.name());
+//!         println!("saw apply to {}", f.name_any());
 //!     }
 //!     Ok(())
 //! }
@@ -98,12 +97,12 @@
 //!
 //! For details, see:
 //!
-//! - [`CustomResource`](crate::CustomResource) for documentation how to configure custom resources
+//! - [`CustomResource`] for documentation how to configure custom resources
 //! - [`runtime::watcher`](crate::runtime::watcher()) for how to long-running watches work and why you want to use this over [`Api::watch`](crate::Api::watch)
-//! - [`runtime`](crate::runtime) for abstractions that help with more complicated Kubernetes application
+//! - [`runtime`] for abstractions that help with more complicated Kubernetes application
 //!
 //! # Examples
-//! A large list of complete, runnable examples with explainations are available in the [examples folder](https://github.com/kube-rs/kube-rs/tree/master/examples).
+//! A large list of complete, runnable examples with explainations are available in the [examples folder](https://github.com/kube-rs/kube/tree/main/examples).
 #![cfg_attr(docsrs, feature(doc_cfg))]
 #![deny(missing_docs)]
 #![forbid(unsafe_code)]
@@ -168,17 +167,22 @@ cfg_error! {
 #[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
 pub use kube_derive::CustomResource;
 
-/// Re-exports from [`kube-runtime`](kube_runtime)
+/// Re-exports from `kube-runtime`
 #[cfg(feature = "runtime")]
 #[cfg_attr(docsrs, doc(cfg(feature = "runtime")))]
 #[doc(inline)]
 pub use kube_runtime as runtime;
 
 pub use crate::core::{CustomResourceExt, Resource, ResourceExt};
-/// Re-exports from [`kube_core`](kube_core)
+/// Re-exports from `kube_core`
 #[doc(inline)]
 pub use kube_core as core;
 
+// Mock tests for the runtime
+#[cfg(test)]
+#[cfg(all(feature = "derive", feature = "runtime"))]
+mod mock_tests;
+
 // Tests that require a cluster and the complete feature set
 // Can be run with `cargo test -p kube --lib --features=runtime,derive -- --ignored`
 #[cfg(test)]
@@ -210,7 +214,7 @@ mod test {
     }
 
     #[tokio::test]
-    #[ignore] // needs kubeconfig
+    #[ignore = "needs kubeconfig"]
     async fn custom_resource_generates_correct_core_structs() {
         use crate::core::{ApiResource, DynamicObject, GroupVersionKind};
         let client = Client::try_default().await.unwrap();
@@ -229,7 +233,7 @@ mod test {
         apiextensions_apiserver::pkg::apis::apiextensions::v1::CustomResourceDefinition,
     };
     #[tokio::test]
-    #[ignore] // needs cluster (creates + patches foo crd)
+    #[ignore = "needs cluster (creates + patches foo crd)"]
     #[cfg(all(feature = "derive", feature = "runtime"))]
     async fn derived_resource_queriable_and_has_subresources() -> Result<(), Box<dyn std::error::Error>> {
         use crate::runtime::wait::{await_condition, conditions};
@@ -243,7 +247,6 @@ mod test {
             .await?;
         let establish = await_condition(crds.clone(), "foos.clux.dev", conditions::is_crd_established());
         let _ = tokio::time::timeout(std::time::Duration::from_secs(10), establish).await?;
-
         // Use it
         let foos: Api<Foo> = Api::default_namespaced(client.clone());
         // Apply from generated struct
@@ -270,12 +273,14 @@ mod test {
                 }
             });
             let o = foos.patch("baz", &ssapply, &Patch::Apply(patch)).await?;
-            assert_eq!(o.spec.replicas, 2);
+            assert_eq!(o.spec.replicas, 2, "patching spec updated spec.replicas");
         }
         // check subresource
         {
-            assert_eq!(foos.get_scale("baz").await?.spec.unwrap().replicas, Some(2));
-            assert!(foos.get_status("baz").await?.status.is_none()); // nothing has set this
+            let scale = foos.get_scale("baz").await?;
+            assert_eq!(scale.spec.unwrap().replicas, Some(2));
+            let status = foos.get_status("baz").await?;
+            assert!(status.status.is_none(), "nothing has set status");
         }
         // set status subresource
         {
@@ -283,7 +288,7 @@ mod test {
             let o = foos
                 .patch_status("baz", &Default::default(), &Patch::Merge(&fs))
                 .await?;
-            assert!(o.status.is_some());
+            assert!(o.status.is_some(), "status set after patch_status");
         }
         // set scale subresource
         {
@@ -291,8 +296,9 @@ mod test {
             let o = foos
                 .patch_scale("baz", &Default::default(), &Patch::Merge(&fs))
                 .await?;
-            assert_eq!(o.status.unwrap().replicas, 1); // something needs to set the status for this
-            assert_eq!(o.spec.unwrap().replicas.unwrap(), 3); // what we asked for got updated
+            assert_eq!(o.status.unwrap().replicas, 1, "scale replicas got patched");
+            let linked_replicas = o.spec.unwrap().replicas.unwrap();
+            assert_eq!(linked_replicas, 3, "patch_scale updates linked spec.replicas");
         }
 
         // cleanup
@@ -303,7 +309,7 @@ mod test {
     }
 
     #[tokio::test]
-    #[ignore] // needs cluster (lists pods)
+    #[ignore = "needs cluster (lists pods)"]
     async fn custom_serialized_objects_are_queryable_and_iterable() -> Result<(), Box<dyn std::error::Error>>
     {
         use crate::core::{
@@ -354,13 +360,13 @@ mod test {
     }
 
     #[tokio::test]
-    #[ignore] // needs cluster (fetches api resources, and lists all)
-    #[cfg(all(feature = "derive"))]
+    #[ignore = "needs cluster (fetches api resources, and lists all)"]
+    #[cfg(feature = "derive")]
     async fn derived_resources_discoverable() -> Result<(), Box<dyn std::error::Error>> {
         use crate::{
             core::{DynamicObject, GroupVersion, GroupVersionKind},
-            discovery::{self, verbs, Discovery, Scope},
-            runtime::wait::{await_condition, conditions},
+            discovery::{self, verbs, ApiGroup, Discovery, Scope},
+            runtime::wait::{await_condition, conditions, Condition},
         };
 
         #[derive(CustomResource, Deserialize, Serialize, Clone, Debug, JsonSchema)]
@@ -376,7 +382,9 @@ mod test {
         crds.patch("testcrs.kube.rs", &ssapply, &Patch::Apply(TestCr::crd()))
             .await?;
         let establish = await_condition(crds.clone(), "testcrs.kube.rs", conditions::is_crd_established());
-        let _ = tokio::time::timeout(std::time::Duration::from_secs(10), establish).await?;
+        let crd = tokio::time::timeout(std::time::Duration::from_secs(10), establish).await??;
+        assert!(conditions::is_crd_established().matches_object(crd.as_ref()));
+        tokio::time::sleep(std::time::Duration::from_secs(2)).await; // Established condition is actually not enough for api discovery :(
 
         // create partial information for it to discover
         let gvk = GroupVersionKind::gvk("kube.rs", "v1", "TestCr");
@@ -386,9 +394,9 @@ mod test {
         let apigroup = discovery::oneshot::pinned_group(&client, &gv).await?;
         let (ar1, caps1) = apigroup.recommended_kind("TestCr").unwrap();
         let (ar2, caps2) = discovery::pinned_kind(&client, &gvk).await?;
-        assert_eq!(caps1.operations.len(), caps2.operations.len());
-        assert_eq!(ar1, ar2);
-        assert_eq!(DynamicObject::api_version(&ar2), "kube.rs/v1");
+        assert_eq!(caps1.operations.len(), caps2.operations.len(), "unequal caps");
+        assert_eq!(ar1, ar2, "unequal apiresource");
+        assert_eq!(DynamicObject::api_version(&ar2), "kube.rs/v1", "unequal dynver");
 
         // run (almost) full discovery
         let discovery = Discovery::new(client.clone())
@@ -398,14 +406,17 @@ mod test {
             .await?;
 
         // check our custom resource first by resolving within groups
-        assert!(discovery.has_group("kube.rs"));
+        assert!(discovery.has_group("kube.rs"), "missing group kube.rs");
         let (ar, _caps) = discovery.resolve_gvk(&gvk).unwrap();
-        assert_eq!(ar.group, gvk.group);
-        assert_eq!(ar.version, gvk.version);
-        assert_eq!(ar.kind, gvk.kind);
+        assert_eq!(ar.group, gvk.group, "unexpected discovered group");
+        assert_eq!(ar.version, gvk.version, "unexcepted discovered ver");
+        assert_eq!(ar.kind, gvk.kind, "unexpected discovered kind");
 
         // check all non-excluded groups that are iterable
-        for group in discovery.groups() {
+        let mut groups = discovery.groups_alphabetical().into_iter();
+        let firstgroup = groups.next().unwrap();
+        assert_eq!(firstgroup.name(), ApiGroup::CORE_GROUP, "core not first");
+        for group in groups {
             for (ar, caps) in group.recommended_resources() {
                 if !caps.supports_operation(verbs::LIST) {
                     continue;
@@ -425,8 +436,8 @@ mod test {
     }
 
     #[tokio::test]
-    #[ignore] // needs cluster (will create await a pod)
-    #[cfg(all(feature = "runtime"))]
+    #[ignore = "needs cluster (will create await a pod)"]
+    #[cfg(feature = "runtime")]
     async fn pod_can_await_conditions() -> Result<(), Box<dyn std::error::Error>> {
         use crate::{
             api::{DeleteParams, PostParams},
@@ -460,7 +471,10 @@ mod test {
         }))?;
 
         let pp = PostParams::default();
-        assert_eq!(data.name(), pods.create(&pp, &data).await?.name());
+        assert_eq!(
+            data.name_unchecked(),
+            pods.create(&pp, &data).await?.name_unchecked()
+        );
 
         // Watch it phase for a few seconds
         let is_running = await_condition(pods.clone(), "busybox-kube4", conditions::is_pod_running());
@@ -504,7 +518,7 @@ mod test {
     }
 
     #[tokio::test]
-    #[ignore] // needs cluster (lists cms)
+    #[ignore = "needs cluster (lists cms)"]
     async fn api_get_opt_handles_404() -> Result<(), Box<dyn std::error::Error>> {
         let client = Client::try_default().await?;
         let api = Api::<ConfigMap>::default_namespaced(client);
